/*
 * flash-tools - a set of tools for flashing handsets
 * Copyright (C) 2005 Jeremy Lain <jeremy.laine@m4x.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <getopt.h>
#include "p2k/flashops.h"

void usage()
{
  printf("Syntax:\n");
  printf("  flash-tool [options]\n\n");
  printf("Options:\n");
  printf("  -b            flash first 8 bytes of the first codegroup\n");
  printf("                after checksums (legacy)\n");
  printf("  -f <catalog>  flash a catalog to a handset\n");
  printf("  -h            display this help message\n");
  printf("  -s <file>     split a Motorola 'one-flash-file', producing\n");
  printf("                a catalog an several binary files\n");
}

int main(const int argc, char* argv[])
{
  struct usb_device *dev = NULL;
  struct usb_device *devs[P2K_DEVICE_MAX];
  size_t nbdevs = 0;
  
  p2k_dev_handle flash_handle;
  char *shxfile = NULL;
  char *catalogfile = NULL;
  size_t cg0break = 0;
  
  int c, i;
  enum { OP_NONE, OP_FLASH, OP_SPLIT } flashop = OP_NONE;
  
  printf("[ This is flash-tool v" VERSION ", a Motorola P2k handset flashing tool ]\n\n");
  while ((c = getopt (argc, argv, "hv:s:f:b")) != -1)
  {
  
    switch (c)
    {
      case 'b':
	cg0break = 8;
	break;
      case 'f':
        flashop = OP_FLASH;
        catalogfile = optarg;
        break;
      case 'h':
        usage();
        return 0;
        break;
      case 's':
        flashop = OP_SPLIT;
        shxfile = optarg;
        break;        
      case 'v':
        //cvalue = optarg;
        break;
    }
  }
  
  switch (flashop)
  {
  
  // flash a catalog to a handset
  case OP_FLASH:
    usb_init();
    usb_find_busses();
    usb_find_devices();

    // find flash device  
    if (!(nbdevs = p2kdev_find(P2K_VENDOR_MOTOROLA, P2K_PRODUCT_SFLASH, &devs[0], P2K_DEVICE_MAX)))
    {    
      printf("Could not find any flash devices, aborting.\n");
      return 1;
    }

    // select the first available flash device
    printf("Found %i flash device(s)\n", nbdevs);
    for(i = 0; i < nbdevs; i++)
    {
      // open flash device
      if (p2kdev_open_sflash(&flash_handle, devs[i]))
      {
	dev = devs[i];
	break;
      }
    }
        
    if (!dev) {
      printf("Could not open USB device, aborting.\n");
      return 1;
    }

    // perform flashing
    if (!flashcat_flash(&flash_handle, catalogfile, cg0break))
    {
      printf("Flashing failed!\n");
      p2kdev_close(&flash_handle);
      return 1;
    }
    
    // close flash device
    p2kdev_close(&flash_handle);  
    break;  
   
  // split a 1FF file   
  case OP_SPLIT:   
    printf("Splitting file '%s' ..\n", shxfile);
    return !flash_split_1ff(shxfile); 
    
  default:
    printf("No action was specified!\n\n");
    usage();
    break;
  }    
  
  return 0;
}
