/*
 * flash-tools - a set of tools for flashing handsets
 * Copyright (C) 2005 Jeremy Lainé <jeremy.laine@m4x.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * The code found below is based on samples found in the manpage of the
 * "select" function in the Linux Programmer's Manual.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "kbdhit.h"
#include <stdio.h>

#ifdef HAVE_CONIO_H
#include <conio.h>
#endif

#ifdef HAVE_TERMIOS_H
#include <termios.h>
#include <sys/time.h>
static struct termios save_termios;
#endif

int tty_cbreak (int fd)
{
#ifdef HAVE_TERMIOS_H
  struct termios ios;

  if (tcgetattr (fd, &save_termios) < 0)
    return -1;

  ios = save_termios;
  ios.c_lflag &= ~(ICANON | ECHO);
  ios.c_cc[VMIN] = 1;
  ios.c_cc[VTIME] = 0;

  return tcsetattr (fd, TCSAFLUSH, &ios);
#else
  return 1;
#endif
}


int tty_reset (int fd)
{
#ifdef HAVE_TERMIOS_H
  return tcsetattr (fd, TCSAFLUSH, &save_termios);
#else
  return 1;
#endif
}


int kbdhit (void)
{
#ifdef HAVE_TERMIOS_H
  fd_set rfds;
  struct timeval tv;

  /* Watch stdin (fd 0) to see when it has input. */
  FD_ZERO (&rfds);
  FD_SET (STDIN_FILENO, &rfds);

  /* Do not wait for data. */
  tv.tv_sec = 0;
  tv.tv_usec = 0;

  /* Must be in raw or cbreak mode for this to work correctly. */
  return select (STDIN_FILENO + 1, &rfds, NULL, NULL, &tv) &&
    FD_ISSET (STDIN_FILENO, &rfds);
#else
  return kbhit();
#endif
}

