/*
 * flash-tools - a set of tools for flashing handsets
 * Copyright (C) 2005 Jeremy Lain <jeremy.laine@m4x.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
#ifndef P2K_P2KDEV_H
#define P2K_P2KDEV_H

#include <usb.h>
#include <stdint.h>

#define P2K_VENDOR_MOTOROLA    0x22b8

#define P2K_PRODUCT_SFLASH     0x1801
#define P2K_PRODUCT_COMPOSITE  0x3001
#define P2K_PRODUCT_MODEM      0x3002

#define P2K_DEVICE_MAX         32
#define P2K_STRING_MAX         256

#define P2K_TIMEOUT           2000
#define P2K_TIMEOUT_LONG      60000

#define FLASH_BLOCK             8192
#define FLASH_INPUT_BUFFER      8192  


//#define DEBUG
#define INFO


typedef struct
{
  usb_dev_handle *usb_handle;
  uint8_t bEndpointIn;
  uint8_t bEndpointOut;
  uint8_t bConfigurationValue;
  uint8_t bInterfaceNumber;  
} p2k_dev_handle;


size_t p2kdev_find(short vendor, short product, struct usb_device *devs[], const size_t maxdevs);

int p2kdev_open(p2k_dev_handle *p2k_handle, struct usb_device *dev);
int p2kdev_open_datalog(p2k_dev_handle *p2k_handle, struct usb_device *dev);
int p2kdev_open_modem(p2k_dev_handle *p2k_handle, struct usb_device *dev);
int p2kdev_open_sflash(p2k_dev_handle *p2k_handle, struct usb_device *dev);

int p2kdev_close(p2k_dev_handle *p2k_handle);

int p2kdev_write(p2k_dev_handle *p2k_handle, char *bytes, const size_t size, const int timeout);
int p2kdev_read(p2k_dev_handle *p2k_handle, char *bytes, const size_t maxsize, const int timeout);

int p2kdev_modem2composite(struct usb_device *dev);

#endif
