/*
 * flash-tools - a set of tools for flashing handsets
 * Copyright (C) 2005 Jeremy Lain <jeremy.laine@m4x.org>
 *
 * base_name function is Copyright (C) 1990, 1998, 1999 Free Software Foundation, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
#include "p2klib.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/timeb.h>

/** Print an array of bytes as hexadecimal, 16 bytes per line.
 *
 * @param bytes
 * @param size
 */
void hexprint(const char* bytes, const int size)
{
  int i;
  
  for (i = 0; i < size; i++)
  {
    if (i && (i % 16) == 0)
    {
      fprintf(stderr, "\n");
    }  
    fprintf(stderr, "%.2x ", (unsigned char)bytes[i]);      
  }
  fprintf(stderr, "\n");
}


/** Convert a single hexadecimal value from its character representation.
 *
 * @param ch the number of bytes to read
 */
unsigned char shx_conv_char(char ch)
{
  if ((ch >= '0') && (ch <= '9'))
  {
    return (ch - '0');
  } else if ((ch >= 'A') && (ch <= 'F')) {
    return 10 + (ch - 'A');
  } else {
    fprintf(stderr, "shx_conv_char : unexpected value : '%c' (%i)\n", ch, ch);
    exit(1);
  }    
}


/** Convert bytes from their character representation.
 *
 * @param chars the characters to read from
 * @param bytes the byte field we write bytes to
 * @param size the number of bytes to read
 */
void shx_conv_chars(const char* chars, char* bytes, const int size)
{
  int i;
  
  for (i = 0; i < size; i++)
  {
    unsigned char bh = shx_conv_char(chars[i*2]);
    unsigned char bl = shx_conv_char(chars[i*2+1]);    
    bytes[i] = (bh << 4) + bl;
  }
}

#ifndef FILESYSTEM_PREFIX_LEN
# define FILESYSTEM_PREFIX_LEN(Filename) 0
#endif

#ifndef ISSLASH
# define ISSLASH(C) ((C) == '/')
#endif


/** In general, we can't use the builtin `basename' function if available,
 *  since it has different meanings in different environments.
 *  In some environments the builtin `basename' modifies its argument.
 *  If NAME is all slashes, be sure to return `/'.
 */
char * base_name (char const *name)
{
  char const *base = name += FILESYSTEM_PREFIX_LEN (name);
  int all_slashes = 1;
  char const *p;

  for (p = name; *p; p++)
    {
      if (ISSLASH (*p))
	base = p + 1;
      else
	all_slashes = 0;
    }

  /* If NAME is all slashes, arrange to return `/'.  */
  if (*base == '\0' && ISSLASH (*name) && all_slashes)
    --base;

  return (char *) base;
}


void root_name(char const *name, char *rootname)
{
  char *fname;
  char *p;
  
  strcpy(rootname, name);
  fname = base_name(rootname);
  
  for (p = fname + strlen(fname) - 1; p > fname; p--)
  {
    if (*p == '.') 
    {
      *p = 0;
      return;
    }
  }  
}


#ifdef _WIN32
int gettimeofday(struct timeval *tvp, struct timezone *tzp)
{
  struct timeb timebuffer;
  
  ftime(&timebuffer);
  if (tvp) {
    tvp->tv_sec = timebuffer.time;
    tvp->tv_usec = timebuffer.millitm * 1000L;
  }
  return (0);
}
#endif


/** Gets the current time.
 */
void timer_time(struct timeval *tstart)
{
  gettimeofday(tstart, NULL);
}


/** Returns the elapsed time in seconds since a given start time.
 */
double timer_elapsed(struct timeval tstart)
{
  struct timeval tcur, result;
  timer_time(&tcur);
  
  /* Perform the carry for the later subtraction by updating y. */
  if (tcur.tv_usec < tstart.tv_usec) {
    int nsec = (tstart.tv_usec - tcur.tv_usec) / 1000000 + 1;
    tstart.tv_usec -= 1000000 * nsec;
    tstart.tv_sec += nsec;
  }
  if (tcur.tv_usec - tstart.tv_usec > 1000000) {
    int nsec = (tcur.tv_usec - tstart.tv_usec) / 1000000;
    tstart.tv_usec += 1000000 * nsec;
    tstart.tv_sec -= nsec;
  }

  /* Compute the time remaining to wait.
     tv_usec is certainly positive. */
  result.tv_sec = tcur.tv_sec - tstart.tv_sec;
  result.tv_usec = tcur.tv_usec - tstart.tv_usec;  
  
  return result.tv_sec + ((double) result.tv_usec / (double) 1000000);
}

